#include "ipc_shm.h"
#include<stdlib.h>
ProcessInfo ProcessInfoFactory(int pid,char *proc_name){
    ProcessInfo proc_info;
    proc_info.p_id=pid;
    strcpy(proc_info.p_name,proc_name);

    return proc_info;
}

int ShmIsExist(){
    if(shmget(SHM_DEFAULT_KEY,sizeof(ProcessDir)+1,IPC_EXCL)>=0){
        //exist
        return 1;
    } else {
        //not exist
        return 0;
    }
}

//reset, return shmid
int ShmInit(int server_pid){
    int shm_id=0;
    ProcessDir *shared_memory;
    if(!ShmIsExist()){
        if((shm_id=shmget(SHM_DEFAULT_KEY,sizeof(ProcessDir)+1,IPC_CREAT | 0644))>=0){
            if ((shared_memory = shmat(shm_id, NULL, 0)) != (ProcessDir *) - 1){
                memset(shared_memory, 0, sizeof(ProcessDir)+1); //shared memory reset
                ProcessDir proc_dir;
                proc_dir.server_pid=server_pid;
                proc_dir.msq_id= MsqCreate();
                proc_dir.proc1= ProcessInfoFactory(-1,"");
                proc_dir.proc2= ProcessInfoFactory(-1,"");
                *shared_memory= proc_dir;
                shmdt(shared_memory);
                return shm_id;
            }
        }
    }
    return -1;

}

int ShmDrop(){
    if(ShmIsExist()){
        if(shmctl(shmget(SHM_DEFAULT_KEY,sizeof(ProcessDir)+1,IPC_EXCL),IPC_RMID, NULL)==0){
            return 1;
        }
    }
    return 0;
}

int GetShmID () {
    return shmget(SHM_DEFAULT_KEY,sizeof(ProcessDir)+1,IPC_EXCL);
}

void PrintProcessStatus () {
    ProcessDir *shared_memory;
    if(ShmIsExist()){
        if ((shared_memory = shmat(GetShmID(), NULL, 0)) != (ProcessDir *) - 1){
            printf("chatting server process pid = \"%d\" is running.\n",shared_memory->server_pid);
            if(shared_memory->proc1.p_id!=-1){
                printf("chatting client process 1 pid = \"%d\" is running.\n",shared_memory->proc1.p_id);
            }
            if(shared_memory->proc2.p_id!=-1){
                printf("chatting client process 2 pid = \"%d\" is running.\n",shared_memory->proc2.p_id);
            }
        }
    } else {
        printf("There are no running processes.\n");
    }

}

// -1: error,
// 0: name error,
// 1: success
int InsertProcessStatus (int pid, char *proc_name) {
    int shm_id;
    ProcessDir *shared_memory;
    if((shm_id=shmget(SHM_DEFAULT_KEY,sizeof(ProcessDir)+1,IPC_CREAT | 0644))>=0){
        if ((shared_memory = shmat(shm_id, NULL, 0)) != (ProcessDir *) - 1){
            if(strcmp(shared_memory->proc1.p_name,proc_name)==0||strcmp(shared_memory->proc2.p_name,proc_name)==0){
                return 0;
            }
            if(shared_memory->proc1.p_id==-1){
                shared_memory->proc1=ProcessInfoFactory(pid,proc_name);
            } else if(shared_memory->proc2.p_id==-1) {
                shared_memory->proc2=ProcessInfoFactory(pid,proc_name);
            } else return -1;
        } else return -1;
        shmdt(shared_memory);
    } else return -1;
    return 1;
}
int RemoveProcessStatus (int pid) {
    int shm_id;
    ProcessDir *shared_memory;
    if((shm_id=shmget(SHM_DEFAULT_KEY,sizeof(ProcessDir)+1,IPC_CREAT | 0644))>=0){
        if ((shared_memory = shmat(shm_id, NULL, 0)) != (ProcessDir *) - 1){
            if(shared_memory->proc1.p_id==pid){
                shared_memory->proc1=ProcessInfoFactory(-1,"");
            } else if(shared_memory->proc2.p_id==pid) {
                shared_memory->proc2=ProcessInfoFactory(-1,"");
            } else return -1;
        } else return -1;
        shmdt(shared_memory);
    } else return -1;
    return 1;
}

//return pid
int SelectProcessPID (char *proc_name) {
    int selected_pid=-1;
    ProcessDir *shared_memory;
    if(ShmIsExist()){
        if ((shared_memory = shmat(GetShmID(), NULL, 0)) != (ProcessDir *) - 1){
            if(!strcmp(proc_name,"server")){
                selected_pid=shared_memory->server_pid;
            } else if(!strcmp(proc_name,shared_memory->proc1.p_name)){
                selected_pid=shared_memory->proc1.p_id;
            } else if(!strcmp(proc_name,shared_memory->proc2.p_name)){
                selected_pid=shared_memory->proc2.p_id;
            }
        }
    }
    else {
        return -1;
    }
    return selected_pid;
}
int SelectMsqId(){
    ProcessDir *shared_memory;
    if(ShmIsExist()){
        if ((shared_memory = shmat(GetShmID(), NULL, 0)) != (ProcessDir *) - 1){
            return shared_memory->msq_id;
        }
    }
    return -1;
}
char* SelectOtherSender(char *sender){
    ProcessDir *shared_memory;
    if ((shared_memory = shmat(GetShmID(), NULL, 0)) != (ProcessDir *) - 1){
        if(!strcmp(sender,shared_memory->proc1.p_name)){
            return shared_memory->proc2.p_name;
        } else if(!strcmp(sender,shared_memory->proc2.p_name)){
            return shared_memory->proc1.p_name;
        }
    }
    return "UNKNOWN";
}
