#include "global.h"
#include <unistd.h>
#include "../ipc_module/ipc_msq.h"
#include "../ipc_module/ipc_shm.h"
void StartupClient(char *client_name);
void ShutdownClient();
void SendMsg(int type,char* sender ,char *data, int status);
int main(int argc,char *argv[]) {

    pid_t forkd_pid = fork();

    if(forkd_pid){
        signal(2,ShutdownClient);
        signal(15,ShutdownClient);
        //parent
        if(argc!=2){
            printf("argument mismatch\n");
            return -1;
        }
        StartupClient(argv[1]);
        //chat input process
        int msg_id1=SelectMsqId();
        int send_proc=1;
        while(send_proc){
            char text[260]={};
            MsgContent send_msg={};
            {
                printf("CHAT> ");
                scanf("%s",text);
            }
            if(strcmp(text,"exit")==0){
                send_proc=0;
                ShutdownClient();
            } else {
                SendMsg(1,argv[1],text,STATUS_CHAT);
            }

        }
        kill(forkd_pid,SIGINT);
    } else {
        //child
        //chat recv process
        int msg_id2=SelectMsqId();
        MsgContent recv_msg={};
        int recv_proc=1;
        while(recv_proc){
            if(msgrcv(msg_id2,&recv_msg,sizeof(MsgContent),0,1)==-1) {
                //error
                printf("error : msgrcv error [msg_id %d, %p]\n",msg_id2,&msg_id2);
                printf("errored buffer data : [%ld %s %s %d]\n",recv_msg.msgtype,recv_msg.sender,recv_msg.data,recv_msg.status);
                recv_proc=0;
            }
            if(strcmp(argv[1], recv_msg.sender) != 0) {
                if(recv_msg.status==STATUS_EXIT){
                    printf("\n[%s] Disconnect.\n",SelectOtherSender(argv[1]));
                } else if(recv_msg.status==STATUS_ENTER) {
                    printf("\n[%s] entered.\n",recv_msg.sender);
                    printf("\nCHAT> ");
                } else {
                    printf("\n[%s]: %s\n",recv_msg.sender,recv_msg.data);
                    printf("\nCHAT> ");
                    fflush(stdout);
                }
            }
        }
    }

}
void SendMsg(int type,char* sender ,char *data, int status){
    MsgContent msg;
    msg.msgtype=type;
    strcpy((char *) msg.sender, sender);
    strcpy((char *) msg.data, data);
    msg.status = status;
    for(int i=0;i<3;i++){
        //server, user, me....
        msgsnd(SelectMsqId(),&msg,sizeof(MsgContent),IPC_NOWAIT);
    }
}
void ShutdownClient() {
    SendMsg(1,"STRANGER","exit",STATUS_EXIT);
    sleep(1);
    if(RemoveProcessStatus(getpid())==-1){
        printf("remove process info from shared memory failed\n");
    }

}
void StartupClient(char *client_name) {
    int startup_status=1;
    switch (InsertProcessStatus(getpid(),client_name)) {
        case 1:
            break;
        case 0:
            printf("The process can not be executed.\nUSER ID already in use.\n");
            startup_status=0;
            break;
        case -1:
            printf("The process can not be executed.\nError\n");
            startup_status=0;
            break;
    }
    if(startup_status==1&&MsqIsExist()){
        //todo:send startup msg
        SendMsg(1,client_name,"startup",STATUS_ENTER);
        printf("  ! Chatting client started !\n");
    } else {
        exit(0);
    }
}
